@@ -27,6 +27,7 @@
 #include <bonobo/bonobo-property-bag-client.h>
 
 enum {
+	PLUG_CREATED,
 	DISCONNECTED,
 	SET_FRAME,
 	ACTIVATE,
@@ -227,11 +228,10 @@ impl_Bonobo_Control_activate (PortableServer_Servant servant,
 	 * if our active state is not what we are changing it to, then
 	 * don't emit the signal
 	 */
-	if (control->priv->active == old_activated)
+	if (control->priv->active != activated)
 		return;
 
-	g_signal_emit (control, control_signals [ACTIVATE], 0, control->priv->active);
-	bonobo_control_activate_notify (control, control->priv->active, ev);
+	g_signal_emit (control, control_signals [ACTIVATE], 0, (gboolean) activated);
 }
 
 static void
@@ -286,6 +286,8 @@ create_plug (BonoboControl *control)
 		gtk_container_add (GTK_CONTAINER (plug),
 				   control->priv->widget);
 
+	g_signal_emit (control, control_signals [PLUG_CREATED], 0);
+
 	g_object_unref (G_OBJECT (plug));
 }
 
@@ -995,6 +997,15 @@ bonobo_control_class_init (BonoboControlClass *klass)
 
 	bonobo_control_parent_class = g_type_class_peek_parent (klass);
 
+	control_signals [PLUG_CREATED] =
+                g_signal_new ("plug_created",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (BonoboControlClass, plug_created),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE, 0);
+
 	control_signals [DISCONNECTED] =
                 g_signal_new ("disconnected",
 			      G_TYPE_FROM_CLASS (object_class),
@@ -1344,6 +1355,18 @@ bonobo_control_set_plug (BonoboControl *control,
 		bonobo_plug_set_control (plug, control);
 }
 
+/**
+ * bonobo_control_get_plug:
+ * @control: the control.a
+ * 
+ * This methods returns the current plug associated with
+ * the control. If the remote container re-parents the
+ * control - the plug will die, and a new plug has to be
+ * created. Thus this should really only be called from
+ * a 'plug_created' signal handler.
+ * 
+ * Return value: the _current_ plug.
+ **/
 BonoboPlug *
 bonobo_control_get_plug (BonoboControl *control)
 {
@@ -1421,15 +1444,15 @@ bonobo_control_get_popup_ui_component (BonoboControl *control)
 }
 
 gboolean
-bonobo_control_do_popup_full (BonoboControl       *control,
+bonobo_control_do_popup_path (BonoboControl       *control,
 			      GtkWidget           *parent_menu_shell,
 			      GtkWidget           *parent_menu_item,
 			      GtkMenuPositionFunc  func,
 			      gpointer             data,
 			      guint                button,
+			      const char          *popup_path,
 			      guint32              activate_time)
 {
-	char      *path;
 	GtkWidget *menu;
 
 	g_return_val_if_fail (BONOBO_IS_CONTROL (control), FALSE);
@@ -1437,15 +1460,11 @@ bonobo_control_do_popup_full (BonoboControl       *control,
 	if (!control->priv->popup_ui_container)
 		return FALSE;
 
-	path = g_strdup_printf ("/popups/button%d", button);
-
 	menu = gtk_menu_new ();
 
 	bonobo_ui_sync_menu_add_popup (
 		BONOBO_UI_SYNC_MENU (control->priv->popup_ui_sync),
-		GTK_MENU (menu), path);
-
-	g_free (path);
+		GTK_MENU (menu), popup_path);
 
 	gtk_menu_set_screen (
 		GTK_MENU (menu),
@@ -1461,6 +1480,29 @@ bonobo_control_do_popup_full (BonoboControl       *control,
 	return TRUE;
 }
 
+gboolean
+bonobo_control_do_popup_full (BonoboControl       *control,
+			      GtkWidget           *parent_menu_shell,
+			      GtkWidget           *parent_menu_item,
+			      GtkMenuPositionFunc  func,
+			      gpointer             data,
+			      guint                button,
+			      guint32              activate_time)
+{
+	char    *path;
+	gboolean retval;
+
+	path = g_strdup_printf ("/popups/button%d", button);
+
+	retval = bonobo_control_do_popup_path
+		(control, parent_menu_shell, parent_menu_item,
+		 func, data, button, path, activate_time);
+
+	g_free (path);
+
+	return retval;
+}
+
 gboolean
 bonobo_control_do_popup (BonoboControl       *control,
 			 guint                button,
